import os
import logging
import json

from typing import Union, Optional, List, Dict, Any

from .api import event, task_event
from .appliance import min_ppa_version
from .exceptions import ValueAlreadyExists

logger = logging.getLogger(__name__)


AnyJson = Union[None, bool, int, float, str, List[Any], Dict[str, Any]]


def _load_json_file(path) -> dict:
    with open(path, "r") as open_file:
        contents = "".join(open_file.readlines())
    try:
        return json.loads(contents)
    except json.JSONDecodeError as e:
        logger.info(f"Failed to load JSON file {path}")
        raise e


def manifest() -> dict:
    return _load_json_file("/.ppa/manifest.json")


@min_ppa_version("2.4.0")
def payload() -> Optional[dict]:
    try:
        return _load_json_file("/.ppa/payload.json")
    except FileNotFoundError:
        return None


@min_ppa_version("2.4.0")
def trigger() -> str:
    return os.environ["PPA_TRIGGER"]


def user() -> str:
    return os.environ["PPA_USER"]


@task_event()
def user_password(text: str, element_id: Optional[str] = None) -> Union[str, None]:
    return event("user/password", text, id=element_id)


@min_ppa_version("2.7.1")
@task_event({409: ValueAlreadyExists})
def set_result_json(json: AnyJson, reason: Optional[str] = "Saving the task result") -> None:
    return event("result/json", reason, message=json)
